///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpz_poly_q.h"

int main(int argc, char *argv[])
{
    fmpz_poly_q_randstate_t R;
    ulong i, N = 1000;

    fmpz_poly_q_randinit(R);    
    srand(123);

    printf("add..."); fflush(stdout);

    /* Aliasing f and h in h = f + g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g, h;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        fmpz_poly_q_init(h);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest(g, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_add(h, f, g);
        fmpz_poly_q_add(f, f, g);
        
        result = fmpz_poly_q_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpz_poly_q_to_string(f);
            strh = fmpz_poly_q_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_poly_q_clear(h);
    }

    /* Aliasing g and h in h = f + g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g, h;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        fmpz_poly_q_init(h);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest(g, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_add(h, f, g);
        fmpz_poly_q_add(g, f, g);
        
        result = fmpz_poly_q_equal(g, h);
        if (!result)
        {
            char * strg, * strh;
            strg = fmpz_poly_q_to_string(g);
            strh = fmpz_poly_q_to_string(h);
            printf("FAIL\n");
            printf("  g = %s\n", strg);
            printf("  h = %s\n", strh);
            free(strg);
            free(strh);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_poly_q_clear(h);
    }

    /* Aliasing f and g in h = f + g */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, h;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(h);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_add(h, f, f);
        fmpz_poly_q_add(f, f, f);
        
        result = fmpz_poly_q_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpz_poly_q_to_string(f);
            strh = fmpz_poly_q_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(h);
    }
    
    /* Verify commutativity */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g, h;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        fmpz_poly_q_init(h);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest(g, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_add(h, f, g);
        fmpz_poly_q_add(f, g, f);
        
        result = fmpz_poly_q_equal(f, h);
        if (!result)
        {
            char * strf, * strh;
            strf = fmpz_poly_q_to_string(f);
            strh = fmpz_poly_q_to_string(h);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  h = %s\n", strh);
            free(strf);
            free(strh);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_poly_q_clear(h);
    }
    
    printf("PASS\n");
    
    fmpz_poly_q_randclear(R);
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

