///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpz_poly_q.h"

int main(int argc, char *argv[])
{
    fmpz_poly_q_randstate_t R;
    ulong i, N = 1000;
    
    fmpz_poly_q_randinit(R);
    srand(123);
    
    printf("div..."); fflush(stdout);
    
    /* Aliasing */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g, q;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        fmpz_poly_q_init(q);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest_not_zero(g, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_div(q, f, g);
        fmpz_poly_q_div(f, f, g);
        
        result = fmpz_poly_q_equal(f, q);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpz_poly_q_to_string(f);
            str2 = fmpz_poly_q_to_string(q);
            printf("FAIL\n");
            printf("  f = %s\n", str1);
            printf("  q = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_poly_q_clear(q);
    }

    /* Check (f / g) * g == f */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g, lhs;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        fmpz_poly_q_init(lhs);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest_not_zero(g, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_div(lhs, f, g);
        fmpz_poly_q_mul(lhs, lhs, g);
        
        result = fmpz_poly_q_equal(lhs, f);
        if (!result)
        {
            char * str1, * str2;
            str1 = fmpz_poly_q_to_string(lhs);
            str2 = fmpz_poly_q_to_string(f);
            printf("FAIL\n");
            printf("  lhs = %s\n", str1);
            printf("  f   = %s\n", str2);
            free(str1);
            free(str2);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_poly_q_clear(lhs);
    }

    printf("PASS\n");
    
    fmpz_poly_q_randclear(R);
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

