///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpz_poly_q.h"

int main(int argc, char *argv[])
{
    fmpz_poly_q_randstate_t R;
    ulong i, N = 1000;
    
    fmpz_poly_q_randinit(R);
    srand(123);
    
    printf("power..."); fflush(stdout);
    
    /* Aliasing f and g in g = f ^ e */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g;
        ulong len = 50, bits = 50, ebits = 10;
        ulong e;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        e = rand() % ebits;
        
        fmpz_poly_q_power(g, f, e);
        fmpz_poly_q_power(f, f, e);
        
        result = fmpz_poly_q_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpz_poly_q_to_string(f);
            strg = fmpz_poly_q_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
    }
    
    /* Compute naively */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g;
        ulong len = 50, bits = 50, ebits = 10;
        ulong e;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        e = rand() % ebits;
        
        if (e == 0)
        {
            fmpz_poly_q_one(g);
        }
        else
        {
            fmpz_poly_q_set(g, f);
            ulong j;
            for (j = 1; j < e; j++)
                fmpz_poly_q_mul(g, g, f);
        }
        
        fmpz_poly_q_power(f, f, e);
        
        result = fmpz_poly_q_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpz_poly_q_to_string(f);
            strg = fmpz_poly_q_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
    }
    
    printf("PASS\n");
    
    fmpz_poly_q_randclear(R);
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

