///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpz_poly_q.h"

int main(int argc, char *argv[])
{
    fmpz_poly_q_randstate_t R;
    ulong i, N = 1000;
    
    fmpz_poly_q_randinit(R);
    srand(123);
    
    printf("set/ equal..."); fflush(stdout);

    /* Equal polynomials */    
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_set(g, f);
        
        result = fmpz_poly_q_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpz_poly_q_to_string(f);
            strg = fmpz_poly_q_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
    }
    
    /* Distinct polynomials */
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f, g;
        fmpz_t x, y;
        ulong len = 50, bits = 50;
        ulong i = rand() % len;
        
        fmpz_poly_q_init(f);
        fmpz_poly_q_init(g);
        
        fmpz_poly_q_randtest(f, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        
        fmpz_poly_q_set(g, f);
        
        x = fmpz_init(g->num->limbs);
        y = fmpz_init(bits / FLINT_BITS + 2);
        
        fmpz_poly_get_coeff_fmpz(x, g->num, i);
        do
            fmpz_randtest(&y, R, rand() % bits);
        while (fmpz_equal(x, y));
        fmpz_poly_set_coeff_fmpz(g->num, i, y);
        
        fmpz_poly_q_canonicalize(g);
        
        result = !fmpz_poly_q_equal(f, g);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpz_poly_q_to_string(f);
            strg = fmpz_poly_q_to_string(g);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpz_poly_q_clear(f);
        fmpz_poly_q_clear(g);
        fmpz_clear(x);
        fmpz_clear(y);
    }
    
    printf("PASS\n");
    
    fmpz_poly_q_randclear(R);
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

