///////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2010 Sebastian Pancratz                                     //
//                                                                           //
// Distributed under the terms of the GNU General Public License as          //
// published by the Free Software Foundation; either version 2 of the        //
// License, or (at your option) any later version.                           //
//                                                                           //
// http://www.gnu.org/licenses/                                              //
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
    extern "C" {
#endif

#include "fmpz_poly_q.h"

int main(int argc, char *argv[])
{
    fmpz_poly_q_randstate_t R;
    ulong i, N = 1000;
    
    fmpz_poly_q_randinit(R);
    srand(123);
    
    printf("swap..."); fflush(stdout);
    
    for (i = 0; i < N; i++)
    {
        int result;
        fmpz_poly_q_t f1, f2, g1, g2;
        ulong len = 50, bits = 50;
        
        fmpz_poly_q_init(f1);
        fmpz_poly_q_init(f2);
        fmpz_poly_q_init(g1);
        fmpz_poly_q_init(g2);
        
        fmpz_poly_q_randtest(f1, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_randtest(g1, R, rand() % len, rand() % bits, rand() % len, rand() % bits);
        fmpz_poly_q_set(f2, f1);
        fmpz_poly_q_set(g2, g1);
        
        fmpz_poly_q_swap(f2, g2);
        
        result = fmpz_poly_q_equal(f1, g2) && fmpz_poly_q_equal(g1, f2);
        if (!result)
        {
            char * strf, * strg;
            strf = fmpz_poly_q_to_string(f1);
            strg = fmpz_poly_q_to_string(g1);
            printf("FAIL\n");
            printf("  f = %s\n", strf);
            printf("  g = %s\n", strg);
            free(strf);
            free(strg);
            abort();
        }
        
        fmpz_poly_q_clear(f1);
        fmpz_poly_q_clear(f2);
        fmpz_poly_q_clear(g1);
        fmpz_poly_q_clear(g2);
    }
    
    printf("PASS\n");
    
    fmpz_poly_q_randclear(R);
    flint_stack_cleanup();
    return EXIT_SUCCESS;
}

#ifdef __cplusplus
    }
#endif

